""" Useful functions for plotting. """

from typing import Tuple

import matplotlib.pyplot as plt
import numpy as np
from sax.typing_ import Model


def plot_model(
    sdict: Model,
    port1: str = "o1",
    ports2: Tuple[str, ...] = None,
    logscale: bool = True,
    fig=None,
    wl_min: float = 1.5,
    wl_max: float = 1.6,
    wl_steps: int = 2000,
    phase: bool = False,
) -> None:
    """Plot Model Sparameters

    Args:
        model: function that returns SDict
        port1: input port name
        ports2: list of ports
        wl_min: wavelength min (um)
        wl_max: wavelength max (um)
        wl_steps: number of wavelength steps
        logscale: plots in dB.
        phase: plots phase instead of magnitude

    .. plot::
        :include-source:

        import gdsfactory.simulation.sax as gs
        import matplotlib.pyplot as plt

        gs.plot_model(gs.models.straight, phase=True, port1="o1")
        plt.show()
    """

    wavelengths = np.linspace(wl_min, wl_max, wl_steps)
    sdict = sdict(wl=wavelengths)

    ports = {ports[0] for ports in sdict.keys()}
    ports2 = ports2 or ports

    if port1 not in ports:
        raise ValueError(f"port1 {port1!r} not in {list(ports)}")

    for port in ports2:
        if port not in ports:
            raise ValueError(f"port2 {port!r} not in {list(ports)}")

    fig = fig or plt.subplot()
    ax = fig.axes

    for port2 in ports2:
        if (port1, port2) in sdict:
            if phase:
                y = np.angle(sdict[(port1, port2)])
                ylabel = "angle (rad)"
            else:
                y = np.abs(sdict[(port1, port2)])
                y = 20 * np.log10(y) if logscale else y
                ylabel = "|S (dB)|" if logscale else "|S|"
            ax.plot(wavelengths * 1e9, y, label=port2)
    ax.set_title(port1)
    ax.set_xlabel("wavelength (nm)")
    ax.set_ylabel(ylabel)
    plt.legend()
    return ax


if __name__ == "__main__":
    import sax

    plot_model(sax.models.straight, phase=True, port1="in2")
    plt.show()



from gdsfactory.simulation.sax import models
from gdsfactory.simulation.sax.from_csv import from_csv
from gdsfactory.simulation.sax.plot_model import plot_model

__all__ = ["from_csv", "plot_model", "models"]



import jax.numpy as jnp
from sax.typing_ import SDict
from sax.utils import reciprocal


def straight(
    *,
    wl: float = 1.55,
    wl0: float = 1.55,
    neff: float = 2.34,
    ng: float = 3.4,
    length: float = 10.0,
    loss: float = 0.0
) -> SDict:
    """Simple straight waveguide model.

    Args:
        wl: wavelength
        wl0: center wavelength
        neff: effective index
        ng: group index
        length: um
        loss: in dB/um

    .. code::

        o1 -------------- o2
                length
    """
    dwl = wl - wl0
    dneff_dwl = (ng - neff) / wl0
    neff = neff - dwl * dneff_dwl
    phase = 2 * jnp.pi * neff * length / wl
    amplitude = jnp.asarray(10 ** (-loss * length / 20), dtype=complex)
    transmission = amplitude * jnp.exp(1j * phase)
    sdict = reciprocal(
        {
            ("o1", "o2"): transmission,
        }
    )
    return sdict


def coupler(*, coupling: float = 0.5) -> SDict:
    r"""Simple coupler model for a single wavelength.

    Args:
        coupler: coupling coefficient

    .. code::

         o2 ________                           ______o3
                    \                         /
                     \        length         /
                      ======================= gap
                     /                       \
            ________/                         \_______
         o1                                          o4

    """
    kappa = coupling ** 0.5
    tau = (1 - coupling) ** 0.5
    sdict = reciprocal(
        {
            ("o1", "o4"): tau,
            ("o1", "o3"): 1j * kappa,
            ("o2", "o4"): 1j * kappa,
            ("o2", "o3"): tau,
        }
    )
    return sdict
